@@ -20,15 +20,17 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import datetime
 
-from util import m18n, m18nc, m18ncE, logWarning, logException, logDebug, SERVERMARK
+from util import m18n, m18nc, m18ncE, logWarning, logException, logDebug
 from sound import Voice, Sound
+from tile import Tile
 from meld import Meld
-from common import InternalParameters, Debug
+from common import Internal, Debug
+from kde import Sorry
 
-# pylint: disable=W0231
+# pylint: disable=super-init-not-called
 # multiple inheritance: pylint thinks ServerMessage.__init__ does not get called.
 # this is no problem: ServerMessage has no __init__ and its parent Message.__init__
-# will be called via the other path thru ClientMessage
+# will be called anyway
 
 class Message(object):
     """those are the message types between client and server. They have no state
@@ -44,17 +46,55 @@ class Message(object):
         # do not use a numerical value because that could easier
         # change with software updates
         Message.defined[self.name] = self
+        className = self.__class__.__name__.replace('Message', '')
+        msgName = self.name.replace(' ', '')
+        assert className ==  msgName, '%s != %s' % ( className, msgName)
 
     def __str__(self):
         return self.name
 
     def __repr__(self):
-        return "<Message: %s>" % self
+        return self.name
+
+    @staticmethod
+    def jelly(key, value):
+        """serialize value for wire transfer. The twisted.pb mechanism with
+        pb.Copyable is too much overhead"""
+        # pylint: disable=too-many-return-statements
+        cls = value.__class__
+        if cls == Tile:
+            return str(value)
+        elif cls == Meld:
+            return value.joined
+        elif isinstance(value, Message):
+            return value.name
+        elif isinstance(value, (list, tuple)):
+            if isinstance(value, tuple) and isinstance(value[0], Message):
+                if value[1] == None or value[1] == []:
+                    return value[0].name
+            return type(value)([Message.jelly(key, x) for x in value])
+        elif isinstance(value, dict):
+            return dict((Message.jelly('key', x[0]), Message.jelly('value', x[1])) for x in value.items())
+        else:
+            if not isinstance(value, (int, long, basestring, float, type(None))):
+                raise Exception('callRemote got illegal arg: %s %s(%s)' % (key, type(value), str(value)))
+            return value
+
+    @staticmethod
+    def jellyAll(args, kwargs):
+        """serialize args and kwargs for wire transfer. The twisted.pb mechanism with
+        pb.Copyable is too much overhead"""
+        args2 = Message.jelly('args', args)
+        kwargs2 = {}
+        for key, value in kwargs.items():
+            kwargs2[key] = Message.jelly(key, value)
+        return args2, kwargs2
 
 class ServerMessage(Message):
     """those classes are used for messages from server to client"""
     # if sendScore is True, this message will send info about player scoring, so the clients can compare
     sendScore = False
+    needsGame = True   # message only applies to an existing game
 
     def clientAction(self, dummyClient, move):
         """default client action: none - this is a virtual class"""
@@ -65,7 +105,6 @@ class ClientMessage(Message):
     def __init__(self, name=None, shortcut=None):
         Message.__init__(self, name, shortcut)
         self.i18nName = m18nc('kajongg', self.name)
-        self.notifyAtOnce = False
 
     def buttonCaption(self):
         """localized, with a & for the shortcut"""
@@ -91,11 +130,29 @@ class ClientMessage(Message):
         table.abort(errMsg)
 
 class NotifyAtOnceMessage(ClientMessage):
-    """those classes are for messages that should pop up at the
-    other clients right away"""
-    def __init__(self, name, shortcut=None):
+    """those classes are for messages from clients that might have to be relayed to the
+    other clients right away.
+
+    Example: If a client says Pung, it sends Message.Pung with the flag 'notifying=True'.
+    This is relayed to the other 3 clients, helping them in their thinking. When the
+    server decides that the Pung is actually to be executed, it sends Message.Pung
+    to all 4 clients, but without 'notifying=True'"""
+
+    sendScore = False
+    def __init__(self, name=None, shortcut=None):
         ClientMessage.__init__(self, name, shortcut)
-        self.notifyAtOnce = True
+
+    def notifyAction(self, dummyClient, move):
+        """the default action for immediate notifications"""
+        move.player.popupMsg(self)
+
+    @classmethod
+    def receivers(cls, request):
+        """who should get the notification? Default is all players except the
+        player who triggered us"""
+        # default: tell all except the source of the notification
+        game = request.block.table.game
+        return list(x for x in game.players if x != request.player)
 
 class PungChowMessage(NotifyAtOnceMessage):
     """common code for Pung and Chow"""
@@ -116,19 +173,19 @@ class PungChowMessage(NotifyAtOnceMessage):
                 self.i18nName))
         dangerousMelds = button.client.maybeDangerous(self)
         if dangerousMelds:
-            lastDiscardName = Meld.tileName(button.client.game.lastDiscard.element)
+            lastDiscard = button.client.game.lastDiscard
             warn = True
             if Debug.dangerousGame and len(dangerousMelds) != len(maySay):
                 button.client.game.debug('only some claimable melds are dangerous: %s' % dangerousMelds)
             if len(dangerousMelds) == 1:
                 txt.append(m18n(
                    'claiming %1 is dangerous because you will have to discard a dangerous tile',
-                   lastDiscardName))
+                   lastDiscard.name()))
             else:
                 for meld in dangerousMelds:
                     txt.append(m18n(
                    'claiming %1 for %2 is dangerous because you will have to discard a dangerous tile',
-                   lastDiscardName, str(meld)))
+                   lastDiscard.name(), str(meld)))
         if not txt:
             txt = [m18n('You may say %1', self.i18nName)]
         return '<br><br>'.join(txt), warn, ''
@@ -168,11 +225,9 @@ class MessageKong(NotifyAtOnceMessage, ServerMessage):
         warn = False
         if myself.originalCall and myself.mayWin:
             warn = True
-            txt.append(m18n('saying Kong for %1 violates Original Call',
-                Meld.tileName(maySay[0])))
+            txt.append(m18n('saying Kong for %1 violates Original Call', Tile(maySay[0][0]).name()))
         if not txt:
-            txt = [m18n('You may say Kong for %1',
-                Meld.tileName(maySay[0][0]))]
+            txt = [m18n('You may say Kong for %1', Tile(maySay[0][0]).name())]
         return '<br><br>'.join(txt), warn, ''
     def clientAction(self, client, move):
         """mirror kong call"""
@@ -220,7 +275,7 @@ class MessageMahJongg(NotifyAtOnceMessage, ServerMessage):
         return m18n('Press here and you win'), False, ''
     def clientAction(self, dummyClient, move):
         """mirror the mahjongg action locally. Check if the balances are correct."""
-        return move.player.declaredMahJongg(move.source, move.withDiscard,
+        return move.player.declaredMahJongg(move.source, move.withDiscardTile,
             move.lastTile, move.lastMeld)
 
 class MessageOriginalCall(NotifyAtOnceMessage, ServerMessage):
@@ -234,11 +289,12 @@ class MessageOriginalCall(NotifyAtOnceMessage, ServerMessage):
         table.clientDiscarded(msg)
     def toolTip(self, button, tile):
         """decorate the action button which will send this message"""
+        assert isinstance(tile, Tile), tile
         myself = button.client.game.myself
-        isCalling = bool((myself.hand - tile.element).callingHands())
+        isCalling = bool((myself.hand - tile).callingHands())
         if not isCalling:
             txt = m18n('discarding %1 and declaring Original Call makes this hand unwinnable',
-                Meld.tileName(tile.element))
+                tile.name())
             return txt, True, txt
         else:
             return (m18n(
@@ -268,17 +324,16 @@ class MessageDiscard(ClientMessage, ServerMessage):
         table.clientDiscarded(msg)
     def toolTip(self, button, tile):
         """decorate the action button which will send this message"""
+        assert isinstance(tile, Tile), tile
         game = button.client.game
         myself = game.myself
         txt = []
         warn = False
         if myself.violatesOriginalCall(tile):
-            txt.append(m18n('discarding %1 violates Original Call',
-                Meld.tileName(tile.element)))
+            txt.append(m18n('discarding %1 violates Original Call', tile.name()))
             warn = True
         if game.dangerousFor(myself, tile):
-            txt.append(m18n('discarding %1 is Dangerous Game',
-                Meld.tileName(tile.element)))
+            txt.append(m18n('discarding %1 is Dangerous Game', tile.name()))
             warn = True
         if not txt:
             txt = [m18n('discard the least useful tile')]
@@ -286,7 +341,7 @@ class MessageDiscard(ClientMessage, ServerMessage):
         return txt, warn, txt
     def clientAction(self, client, move):
         """execute the discard locally"""
-        if client.isHumanClient() and InternalParameters.field:
+        if client.isHumanClient() and Internal.field:
             move.player.handBoard.setEnabled(False)
         move.player.speak(move.tile)
         return client.game.hasDiscarded(move.player, move.tile)
@@ -294,25 +349,52 @@ class MessageDiscard(ClientMessage, ServerMessage):
 class MessageProposeGameId(ServerMessage):
     """the game server proposes a new game id. We check if it is available
     in our local data base - we want to use the same gameid everywhere"""
+    needsGame = False
     def clientAction(self, client, move):
         """ask the client"""
         # move.source are the players in seating order
         # we cannot just use table.playerNames - the seating order is now different (random)
         return client.reserveGameId(move.gameid)
 
+class MessageTableChanged(ServerMessage):
+    """somebody joined or left a table"""
+    needsGame = False
+    def clientAction(self, client, move):
+        """update our copy"""
+        return client.tableChanged(move.source)
+
 class MessageReadyForGameStart(ServerMessage):
     """the game server asks us if we are ready for game start"""
+    needsGame = False
     def clientAction(self, client, move):
         """ask the client"""
-        def hideTableList(dummy):
+        def hideTableList(result):
             """hide it only after player says I am ready"""
-            if client.tableList:
+            if result == Message.OK and client.tableList and client.tableList.isVisible():
+                if Debug.table:
+                    logDebug('%s hiding table list because game started' % client.name)
                 client.tableList.hide()
+            return result
         # move.source are the players in seating order
         # we cannot just use table.playerNames - the seating order is now different (random)
         return client.readyForGameStart(move.tableid, move.gameid,
             move.wantedGame, move.source, shouldSave=move.shouldSave).addCallback(hideTableList)
 
+class MessageNoGameStart(NotifyAtOnceMessage):
+    """the client says he does not want to start the game after all"""
+    needsGame = False
+    def notifyAction(self, client, move):
+        if client.beginQuestion or client.game:
+            Sorry(m18n('%1 is not ready to start the game', move.player.name))
+        if client.beginQuestion:
+            client.beginQuestion.cancel()
+        elif client.game:
+            return client.game.close()
+    @classmethod
+    def receivers(cls, request):
+        """notification is not needed for those who already said no game"""
+        return list(x.player for x in request.block.requests if x.answer != Message.NoGameStart)
+
 class MessageReadyForHandStart(ServerMessage):
     """the game server asks us if we are ready for a new hand"""
     def clientAction(self, client, move):
@@ -325,8 +407,9 @@ class MessageInitHand(ServerMessage):
         """prepare a new hand"""
         client.game.divideAt = move.divideAt
         client.game.wall.divide()
-        client.shutdownClients(exception=client)
-        field = InternalParameters.field
+        if hasattr(client,'shutdownHumanClients'):
+            client.shutdownHumanClients(exception=client)
+        field = Internal.field
         if field:
             field.setWindowTitle(m18n('Kajongg <numid>%1</numid>', client.game.handId()))
             field.discardBoard.setRandomPlaces(client.game.randomGenerator)
@@ -336,13 +419,13 @@ class MessageSetConcealedTiles(ServerMessage):
     """the game server assigns tiles to player"""
     def clientAction(self, client, move):
         """set tiles for player"""
-        return client.game.setConcealedTiles(move.source)
+        return move.player.addConcealedTiles(client.game.wall.deal(move.tiles), animated=False)
 
 class MessageShowConcealedTiles(ServerMessage):
     """the game server assigns tiles to player"""
     def clientAction(self, dummyClient, move):
         """set tiles for player"""
-        return move.player.showConcealedTiles(move.source, move.show)
+        return move.player.showConcealedTiles(move.tiles, move.show)
 
 class MessageSaveHand(ServerMessage):
     """the game server tells us to save the hand"""
@@ -350,28 +433,23 @@ class MessageSaveHand(ServerMessage):
         """save the hand"""
         return client.game.saveHand()
 
-class MessagePopupMsg(ServerMessage):
-    """the game server tells us to show a popup for a player"""
-    def clientAction(self, dummyClient, move):
-        """popup the message"""
-        return move.player.popupMsg(move.msg)
-
 class MessageAskForClaims(ServerMessage):
     """the game server asks us if we want to claim a tile"""
     def clientAction(self, client, move):
         """ask the player"""
-        if not client.thatWasMe(move.player):
-            return client.ask(move, [Message.NoClaim, Message.Chow, Message.Pung, Message.Kong, Message.MahJongg])
+        if client.thatWasMe(move.player):
+            raise Exception('Server asked me(%s) for claims but I just discarded that tile!' % move.player)
+        return client.ask(move, [Message.NoClaim, Message.Chow, Message.Pung, Message.Kong, Message.MahJongg])
 
 class MessagePickedTile(ServerMessage):
     """the game server tells us who picked a tile"""
     def clientAction(self, client, move):
         """mirror the picked tile"""
-        assert move.player.pickedTile(move.deadEnd, tileName=move.source).element == move.source, \
-            (move.player.lastTile, move.source)
+        assert move.player.pickedTile(move.deadEnd, tileName=move.tile) == move.tile, \
+            (move.player.lastTile, move.tile)
         if client.thatWasMe(move.player):
-            if move.source[0] in 'fy':
-                return Message.Bonus, move.source
+            if move.tile.isBonus():
+                return Message.Bonus, move.tile
             else:
                 return client.myAction(move)
 
@@ -381,11 +459,13 @@ class MessageActivePlayer(ServerMessage):
         """set the active player"""
         client.game.activePlayer = move.player
 
-class MessageViolatedOriginalCall(ServerMessage):
+class MessageViolatesOriginalCall(ServerMessage):
     """the game server tells us who violated an original call"""
+    def __init__(self):
+        ServerMessage.__init__(self, name=m18ncE('kajongg', 'Violates Original Call'))
     def clientAction(self, client, move):
         """violation: player may not say mah jongg"""
-        move.player.popupMsg(m18n('Violates Original Call'))
+        move.player.popupMsg(self)
         move.player.mayWin = False
         if Debug.originalCall:
             logDebug('%s: cleared mayWin' % move.player)
@@ -449,23 +529,23 @@ class MessageDeclaredKong(ServerMessage):
         """mirror the action locally"""
         prompts = None
         if not client.thatWasMe(move.player):
-            if len(move.source) != 4 or move.source[0].istitle():
+            if len(move.tiles) != 4 or move.tiles[0].istitle():
                 # do not do this when adding a 4th tile to an exposed pung
-                move.player.showConcealedTiles(move.source)
+                move.player.showConcealedTiles(move.tiles)
             else:
-                move.player.showConcealedTiles(move.source[3:4])
+                move.player.showConcealedTiles(move.tiles[3:4])
             prompts = [Message.NoClaim, Message.MahJongg]
-        move.exposedMeld = move.player.exposeMeld(move.source)
+        move.exposedMeld = move.player.exposeMeld(move.tiles)
         if prompts:
             return client.ask(move, prompts)
 
-class MessageRobbedTheKong(ServerMessage):
+class MessageRobbedTheKong(NotifyAtOnceMessage, ServerMessage):
     """the game server tells us who robbed the kong"""
     def clientAction(self, client, move):
         """mirror the action locally"""
         prevMove = client.game.lastMoves(only=[Message.DeclaredKong]).next()
-        prevKong = Meld(prevMove.source)
-        prevMove.player.robTile(prevKong.pairs[0].capitalize())
+        prevKong = Meld(prevMove.tiles)
+        prevMove.player.robTile(prevKong[0].upper())
         move.player.lastSource = 'k'
         client.game.addCsvTag('robbedKong', forAllPlayers=True)
 
@@ -473,43 +553,42 @@ class MessageCalling(ServerMessage):
     """the game server tells us who announced a calling hand"""
     def clientAction(self, client, move):
         """tell user and save this information locally"""
-        move.player.popupMsg(m18n('Calling'))
+        move.player.popupMsg(self)
         move.player.isCalling = True
-        # otherwise we have a visible artifact of the discarded tile.
-        # Only when animations are disabled. Why?
-        if InternalParameters.field:
-            InternalParameters.field.centralView.resizeEvent(None)
         return client.ask(move, [Message.OK])
 
-class MessagePlayedDangerous(ServerMessage):
+class MessageDangerousGame(ServerMessage):
     """the game server tells us who played dangerous game"""
+    def __init__(self):
+        ServerMessage.__init__(self, name=m18ncE('kajongg', 'Dangerous Game'))
     def clientAction(self, client, move):
         """mirror the dangerous game action locally"""
-        move.player.popupMsg(m18n('Dangerous Game'))
+        move.player.popupMsg(self)
         move.player.playedDangerous = True
         return client.ask(move, [Message.OK])
 
-class MessageHasNoChoice(ServerMessage):
+class MessageNoChoice(ServerMessage):
     """the game server tells us who had no choice avoiding dangerous game"""
     def __init__(self):
-        ServerMessage.__init__(self)
+        ServerMessage.__init__(self, name=m18ncE('kajongg', 'No Choice'))
         self.move = None
 
     def clientAction(self, client, move):
         """mirror the no choice action locally"""
         self.move = move
-        move.player.popupMsg(m18n('No Choice'))
+        move.player.popupMsg(self)
         move.player.claimedNoChoice = True
-        move.player.showConcealedTiles(move.tile)
+        move.player.showConcealedTiles(move.tiles)
         # otherwise we have a visible artifact of the discarded tile.
         # Only when animations are disabled. Why?
-        if InternalParameters.field:
-            InternalParameters.field.centralView.resizeEvent(None)
+        if Internal.field:
+            Internal.field.centralView.resizeEvent(None)
         return client.ask(move, [Message.OK]).addCallback(self.hideConcealedAgain)
 
-    def hideConcealedAgain(self, dummyResult):
+    def hideConcealedAgain(self, result):
         """only show them for explaining the 'no choice'"""
-        self.move.player.showConcealedTiles(self.move.tile, False)
+        self.move.player.showConcealedTiles(self.move.tiles, False)
+        return result
 
 class MessageUsedDangerousFrom(ServerMessage):
     """the game server tells us somebody claimed a dangerous tile"""
@@ -526,6 +605,7 @@ class MessageDraw(ServerMessage):
 
 class MessageError(ServerMessage):
     """a client errors"""
+    needsGame = False
     def clientAction(self, dummyClient, move):
         """show the error message from server"""
         return logWarning(move.source)
@@ -554,29 +634,34 @@ class MessageNoClaim(NotifyAtOnceMessage, ServerMessage):
         """returns text and warning flag for button and text for tile for button and text for tile"""
         return m18n('You cannot or do not want to claim this tile'), False, ''
 
+    @classmethod
+    def receivers(cls, request):
+        """no Claim notifications are not needed for those who already answered"""
+        return list(x.player for x in request.block.requests if x.answer is None)
+
 def __scanSelf():
     """for every message defined in this module which can actually be used for traffic,
     generate a class variable Message.msg where msg is the name (without spaces)
     of the message. Example: 'Message.NoClaim'.
-    Those will be used as stateless constants. Also add them to dict Message.defined."""
+    Those will be used as stateless constants. Also add them to dict Message.defined, but with spaces."""
     if not Message.defined:
         for glob in globals().values():
             if hasattr(glob, "__mro__"):
                 if glob.__mro__[-2] == Message and len(glob.__mro__) > 2:
                     if glob.__name__.startswith('Message'):
-                        msg = glob()
+                        try:
+                            msg = glob()
+                        except Exception:
+                            print('cannot instantiate %s' % glob.__name__)
+                            raise
                         type.__setattr__(Message, msg.name.replace(' ', ''), msg)
 
-class ChatMessage:
+class ChatMessage(object):
     """holds relevant info about a chat message"""
     def __init__(self, tableid, fromUser=None, message=None, isStatusMessage=False):
-        if isinstance(tableid, basestring) and SERVERMARK in tableid:
-            parts = tableid.split(SERVERMARK)
-            self.tableid = int(parts[0])
-            self.timestamp = datetime.time(hour=int(parts[1]), minute=int(parts[2]), second=int(parts[3]))
-            self.fromUser = parts[4]
-            self.message = parts[5]
-            self.isStatusMessage = bool(int(parts[6]))
+        if isinstance(tableid, tuple):
+            self.tableid, hour, minute, second, self.fromUser, self.message, self.isStatusMessage = tableid
+            self.timestamp = datetime.time(hour=hour, minute=minute, second=second)
         else:
             self.tableid = tableid
             self.fromUser = fromUser
@@ -604,15 +689,9 @@ class ChatMessage:
     def __repr__(self):
         return unicode(self)
 
-    def serialize(self):
-        """encode me in a string for network transfer"""
-        return SERVERMARK.join([
-            str(self.tableid),
-            str(self.timestamp.hour),
-            str(self.timestamp.minute),
-            str(self.timestamp.second),
-            self.fromUser,
-            self.message,
-            str(int(self.isStatusMessage))])
+    def asList(self):
+        """encode me for network transfer"""
+        return (self.tableid, self.timestamp.hour, self.timestamp.minute, self.timestamp.second,
+            self.fromUser, self.message, self.isStatusMessage)
 
 __scanSelf()
