@@ -18,19 +18,20 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-import os, tarfile, subprocess, datetime, cStringIO
+import os, tarfile, subprocess, datetime
+from io import BytesIO
 from hashlib import md5
 if os.name == 'nt':
-    import winsound # pylint: disable=F0401
+    import winsound # pylint: disable=import-error
 
-from common import Debug, InternalParameters
+from common import Debug, Internal
 from util import which, logWarning, m18n, cacheDir, logDebug, \
     removeIfExists, logException, uniqueList
 
-if InternalParameters.haveKDE:
+if Internal.haveKDE:
     from kde import KGlobal, KConfigGroup
 
-from meld import Meld
+from tile import Tile
 
         # Phonon does not work with short files - it plays them
         # simultaneously or only parts of them. Mar 2010, KDE 4.4. True for mp3
@@ -86,7 +87,7 @@ class Sound(object):
                 except OSError:
                     pass
                 if Debug.sound:
-                    game = InternalParameters.field.game
+                    game = Internal.field.game
                     game.debug('10 seconds passed. Killing %s' % process.name)
             else:
                 remaining.append(process)
@@ -97,8 +98,8 @@ class Sound(object):
         """this is what the user of this module will call."""
         if not Sound.enabled:
             return
-        game = InternalParameters.field.game
-        reactor = InternalParameters.reactor
+        game = Internal.field.game
+        reactor = Internal.reactor
         if game and not game.autoPlay and Sound.playProcesses:
             # in normal play, wait a moment between two speaks. Otherwise
             # sometimes too many simultaneous speaks make them ununderstandable
@@ -141,7 +142,7 @@ class Sound(object):
             # getTalkerCodes returns nothing
             # this all feels immature
             if len(text) == 2 and text[0] in 'sdbcw':
-                text = Meld.tileName(text)
+                text = Tile(text).name()
             args = ['qdbus', 'org.kde.jovie',
                 '/KSpeech', 'say', text, '1']
             subprocess.Popen(args)
@@ -193,7 +194,7 @@ class Voice(object):
     @staticmethod
     def availableVoices():
         """a list of all voice directories"""
-        if not Voice.__availableVoices and InternalParameters.haveKDE:
+        if not Voice.__availableVoices and Internal.haveKDE:
             result = []
             for parentDirectory in KGlobal.dirs().findDirs("appdata", "voices"):
                 parentDirectory = unicode(parentDirectory)
@@ -305,7 +306,7 @@ class Voice(object):
                     logDebug('md5sum %s changed, rewriting %s with %s' % (existingMd5sum, md5Name, self.__md5sum))
             try:
                 open(md5Name, 'w').write('%s\n' % self.__md5sum)
-            except BaseException, exception:
+            except BaseException as exception:
                 logException(m18n('cannot write <filename>%1</filename>: %2', md5Name, str(exception)))
         if archiveExists:
             archiveIsOlder = os.path.getmtime(md5Name) > os.path.getmtime(self.archiveName())
@@ -334,14 +335,11 @@ class Voice(object):
         if os.path.exists(self.md5FileName()):
             return open(self.md5FileName(), 'r').readlines()[0].strip()
 
-    @apply
-    def md5sum():
+    @property
+    def md5sum(self):
         """the current checksum over all ogg files"""
-        def fget(self):
-            # pylint: disable=W0212
-            self.__computeMd5sum()
-            return self.__md5sum
-        return property(**locals())
+        self.__computeMd5sum()
+        return self.__md5sum
 
     def __setArchiveContent(self, content):
         """fill the Voice with ogg files"""
@@ -349,7 +347,7 @@ class Voice(object):
             return
         if not os.path.exists(self.directory):
             os.makedirs(self.directory)
-        filelike = cStringIO.StringIO(content)
+        filelike = BytesIO(content)
         tarFile = tarfile.open(mode='r|bz2', fileobj=filelike)
         tarFile.extractall(path=self.directory)
         if Debug.sound:
@@ -357,16 +355,14 @@ class Voice(object):
         tarFile.close()
         filelike.close()
 
-    @apply
-    def archiveContent():
+    @property
+    def archiveContent(self):
         """the content of the tarfile"""
-        def fget(self):
-            # pylint: disable=W0212
-            self.__buildArchive()
-            if os.path.exists(self.archiveName()):
-                return open(self.archiveName()).read()
-        def fset(self, content):
-            # pylint: disable=W0212
-            self.__setArchiveContent(content)
+        self.__buildArchive()
+        if os.path.exists(self.archiveName()):
+            return open(self.archiveName()).read()
 
-        return property(**locals())
+    @archiveContent.setter
+    def archiveContent(self, content):
+        """new archive content"""
+        self.__setArchiveContent(content)
