@@ -3,10 +3,6 @@
 """
 Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-The function isAlive() is taken from the book
-"Rapid GUI Programming with Python and Qt"
-by Mark Summerfield.
-
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
@@ -39,11 +35,9 @@ SERVERMARK = '&&SERVER&&'
 
 # util must not import twisted or we need to change kajongg.py
 
-import sip
+from common import Options, Internal, Debug
 
-from common import InternalParameters
-
-if InternalParameters.haveKDE:
+if Internal.haveKDE:
     from kde import i18n, i18nc, Sorry, Information, NoPrompt
 else:
     # a server might not have KDE4
@@ -63,18 +57,18 @@ else:
         """dummy for server"""
         return i18n(englishIn, *args)
 
-if not InternalParameters.isServer:
+if not Internal.isServer:
     from kde import KGlobal
 else:
-    class PrintFirstArg:
+    class PrintFirstArg(object):
         """just print the first argument"""
         def __init__(self, *args):
             kprint(args[0])
-    Sorry = Information = PrintFirstArg  # pylint: disable=C0103
+    Sorry = Information = PrintFirstArg  # pylint: disable=invalid-name
 
 def appdataDir():
     """the per user directory with kajongg application information like the database"""
-    if InternalParameters.isServer:
+    if Internal.isServer:
         # the server might or might not have KDE installed, so to be on
         # the safe side we use our own .kajonggserver directory
         # the following code moves an existing kajonggserver.db to .kajonggserver
@@ -90,7 +84,10 @@ def appdataDir():
             shutil.move(oldPath, newPath)
             logInfo('moved %s to %s' % (oldPath,  newPath))
         if not os.path.exists(newPath):
-            os.makedirs(newPath)
+            try:
+                os.makedirs(newPath)
+            except OSError:
+                pass
         return newPath
     else:
         result = os.path.dirname(unicode(KGlobal.dirs().locateLocal("appdata", ""))) + '/'
@@ -98,13 +95,16 @@ def appdataDir():
 
 def cacheDir():
     """the cache directory for this user"""
-    if InternalParameters.isServer:
+    if Internal.isServer:
         result = os.path.join(appdataDir(), 'cache')
     else:
         result = os.path.dirname(unicode(KGlobal.dirs().locateLocal("cache", "")))
         result = os.path.join(result, 'kajongg')
     if not os.path.exists(result):
-        os.makedirs(result)
+        try:
+            os.makedirs(result)
+        except OSError:
+            pass
     return result
 
 ENGLISHDICT = {}
@@ -135,7 +135,7 @@ def stack(msg, limit=6):
 
 def initLog(logName):
     """init the loggers"""
-    global LOGGER # pylint: disable=W0603
+    global LOGGER # pylint: disable=global-statement
     LOGGER = logging.getLogger(logName)
     try:
         handler = logging.handlers.SysLogHandler('/dev/log')
@@ -173,6 +173,7 @@ def xToUtf8(msg, args=None):
 
 def logMessage(msg, prio, showDialog, showStack=False, withGamePrefix=True):
     """writes info message to log and to stdout"""
+    # pylint: disable=R0912
     if isinstance(msg, Exception):
         msg = ' '.join(unicode(x.decode(getpreferredencoding()) \
              if isinstance(x, str) else unicode(x)) for x in msg.args if x is not None)
@@ -182,14 +183,21 @@ def logMessage(msg, prio, showDialog, showStack=False, withGamePrefix=True):
         msg = unicode(str(msg), 'utf-8')
     msg = translateServerMessage(msg)
     logMsg = msg
-    if withGamePrefix and InternalParameters.logPrefix:
-        logMsg = '%s: %s' % (InternalParameters.logPrefix, msg)
+    if withGamePrefix and Internal.logPrefix:
+        if Debug.process:
+            logMsg = '%s%d: %s' % (Internal.logPrefix, os.getpid(), msg)
+        else:
+            logMsg = '%s: %s' % (Internal.logPrefix, msg)
     __logUnicodeMessage(prio, logMsg)
     if showStack:
-        for line in traceback.format_stack()[2:-3]:
+        if showStack is True:
+            lower = 2
+        else:
+            lower = -showStack - 3
+        for line in traceback.format_stack()[lower:-3]:
             if not 'logException' in line:
                 __logUnicodeMessage(prio, '  ' + line.strip())
-    if InternalParameters.hasGUI and showDialog:
+    if showDialog:
         if prio == logging.INFO:
             return Information(msg)
         else:
@@ -248,24 +256,13 @@ def m18ncE(dummyContext, englishText):
     """use this if you want to get the english text right now but still have the string translated"""
     return englishText
 
-def isAlive(qobj):
-    """is the underlying C++ object still valid?"""
-    if qobj is None:
-        return False
-    try:
-        sip.unwrapinstance(qobj)
-    except RuntimeError:
-        return False
-    else:
-        return True
-
 def socketName():
     """client and server process use this socket to talk to each other"""
     serverDir = os.path.expanduser('~/.kajonggserver')
     if not os.path.exists(serverDir):
         appdataDir() # allocate the directory and possibly move old databases there
-    if InternalParameters.socket:
-        return InternalParameters.socket
+    if Options.socket:
+        return Options.socket
     else:
         return os.path.join(serverDir, 'socket')
 
@@ -330,10 +327,10 @@ def kprint(*args, **kwargs):
             arg = repr(arg)
         arg = arg.encode(STDOUTENCODING, 'ignore')
         newArgs.append(arg)
-    # we need * magic: pylint: disable=W0142
+    # we need * magic: pylint: disable=star-args
     try:
         print(*newArgs, sep=kwargs.get('sep', ' '), end=kwargs.get('end', '\n'), file=kwargs.get('file'))
-    except IOError, exception:
+    except IOError as exception:
         # very big konsole, busy system: sometimes Python says
         # resource temporarily not available
         time.sleep(0.1)
@@ -363,3 +360,37 @@ class Duration(object):
                 logException(msg)
             else:
                 logDebug(msg)
+
+def checkMemory():
+    """as the name says"""
+    #pylint: disable=too-many-branches
+    if not Debug.gc:
+        return
+    gc.set_threshold( 0 )
+    gc.set_debug( gc.DEBUG_LEAK )
+    gc.enable()
+    logDebug('collecting {{{')
+    gc.collect()        # we want to eliminate all output
+    logDebug('}}} done')
+
+    # code like this may help to find specific things
+    if True:
+        interesting = ('Client', 'Player', 'Game')
+        for obj in gc.garbage:
+            if hasattr(obj, 'cell_contents'):
+                obj = obj.cell_contents
+            if not any(x in repr(obj) for x in interesting):
+                continue
+            for referrer in gc.get_referrers(obj):
+                if referrer is gc.garbage:
+                    continue
+                if hasattr(referrer, 'cell_contents'):
+                    referrer = referrer.cell_contents
+                if referrer.__class__.__name__ in interesting:
+                    for referent in gc.get_referents(referrer):
+                        logDebug('%s refers to %s' % (referrer, referent))
+                else:
+                    logDebug('referrer of %s/%s is: id=%s type=%s %s' % (
+                        type(obj), obj, id(referrer), type(referrer), referrer))
+    logDebug('unreachable:%s' % gc.collect())
+    gc.set_debug(0)
